#include <iostream>
#include <fstream>
#include <string>

#include "json/json.h"

#include "boost/uuid/uuid.hpp"
#include "boost/uuid/uuid_generators.hpp"
#include "boost/uuid/uuid_io.hpp"

using json = Json::Value;
using namespace boost;

template<typename T>
json createUUIDArray(const T& arr)
{
  json result;
  int ii = 0;
  result.resize(arr.size());
  for (auto ait = arr.begin(); ait != arr.end(); ++ait, ++ii)
  {
    result[ii] = uuids::to_string(*ait);
  }
  return result;
}

uuids::uuid asUUID(const json& entry)
{
  uuids::string_generator gen;
  return gen(entry.asString());
}

int main(int argc, char* argv[])
{
  json j;
  // No static initialization. Must write klunky c++
  // Cannot create a JSON array directly; it is implied by use of integer argument to the [] operator:
  json arr;
  arr[0] = 1;
  arr[1] = 2;
  arr[2] = 3;
  // Cannot create a JSON object directly; it is implied by use of string arguments to the [] operator:
  j["foo"] = arr;
  j["bar"]["baz"] = 3.141;
  j["bar"]["xyz"] = "zy";

  auto gen = new uuids::basic_random_generator<boost::mt19937>;
  std::vector<uuids::uuid> uids;

  auto uid = (*gen)();
  uids.push_back(uid);
  uid = (*gen)();
  uids.push_back(uid);

  // Must manually convert to a string (or something else) here:
  j["bar"]["xyz"] = uuids::to_string(uid);
  // Must write+call helper to convert to JSON:
  j["bar"]["quux"] = createUUIDArray(uids);
  std::cout << j << "\n";
  json r;
  r["fee"] = false;
  r["fi"] = true;
  r["fo"] = "fum";
  // Must write+call helper to convert from JSON:
  uid = asUUID(j["bar"]["quux"][1]);
  r["fum"] = uuids::to_string(uid);

  // Blark! Must manually create a writer object to avoid pretty-printing:
  Json::FastWriter wri;
  std::cout << wri.write(r) << "\n";
  return 0;
}
