#include <iostream>
#include <fstream>
#include <string>

#include "nlohmann/json.hpp"
#include "boost/uuid/uuid.hpp"
#include "boost/uuid/uuid_generators.hpp"
#include "boost/uuid/uuid_io.hpp"

using namespace nlohmann;
using namespace boost;

// We need not, but can, write a helper for converting JSON to/from UUIDs.
// If we don't override, they appear as JSON arrays of unsigned integers.
namespace nlohmann
{
  template<>
  struct adl_serializer<boost::uuids::uuid> {
    static void to_json(json& j, const boost::uuids::uuid& opt) {
      if (opt.is_nil()) {
        j = nullptr;
      } else {
        j = boost::uuids::to_string(opt);
      }
    }

    static void from_json(const json& j, boost::uuids::uuid& opt) {
      if (j.is_null()) {
        boost::uuids::nil_generator gen;
        opt = gen();
      } else {
        boost::uuids::string_generator gen;
        opt = gen(j.get<std::string>());
      }
    }
  };
}

int main(int argc, char* argv[])
{
  // Fancy initialization means we can construct
  // complex JSON objects containing variable values
  // without (a) creating a JSON string and parsing it
  // or (b) calling a bunch of C++ methods.
  json j = {
    { "foo", {1, 2, 3} },
    { "bar", {
        {"baz", 3.141},
        {"xyz", "zy"}
      }
    }
  };

  auto gen = new uuids::basic_random_generator<boost::mt19937>;
  std::vector<uuids::uuid> uids;
  auto uid = (*gen)();
  uids.push_back(uid);
  uid = (*gen)();
  uids.push_back(uid);

  // No need to call helper functions; it is implicit, for both a single value and a vector:
  j["bar"]["xyz"] = uid;
  j["bar"]["quux"] = uids;
  std::cout << j << "\n";
  json r = { {"fee", false}, {"fi", true}, {"fo", "fum"} };
  // Also, conversion the other direction is performed on assignment:
  uid = j["bar"]["quux"][1];
  r["fum"] = uid;
  std::cout << r << "\n"; // Could also call r.dump(2) to get pretty-printed string
  return 0;
}
